﻿using System;

using DevExpress.Utils;
using DevExpress.XtraEditors;
using DevExpress.XtraGrid.Views.Grid;

namespace TestProject
{
    /// <summary>
    /// 메인 폼
    /// </summary>
    public partial class MainForm : XtraForm
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 생성자 - MainForm()

        /// <summary>
        /// 생성자
        /// </summary>
        public MainForm()
        {
            InitializeComponent();

            this.gridView.InitializeView
            (
                true,                           // Multi Select
                GridMultiSelectMode.CellSelect, // Grid Multi Select Mode
                DrawFocusRectStyle.CellFocus,   // Draw Focus Rect Style
                true,                           // Show Indicator
                true,                           // Show Column Headers
                true,                           // Allow Column Moving
                true,                           // Allow Column Resizing
                true,                           // Allow Filter
                true,                           // Allow Sort
                false,                          // Allow Cell Merge
                EditorShowMode.Default,         // Editor Show Mode
                true                            // Editable
            );

            this.gridView.OptionsBehavior.AutoPopulateColumns = true;

            this.gridView.OptionsBehavior.AutoExpandAllGroups = true;

            this.gridView.OptionsView.ShowGroupPanel = true;

            this.gridView.OptionsView.ShowGroupedColumns = true;

            this.gridControl.UseEmbeddedNavigator = true;

            this.gridControl.DataSource = new nwindDataSetTableAdapters.OrdersTableAdapter().GetData();

            this.gridView.Columns["CustomerID"].GroupIndex = 0;

            this.gridView.BestFitColumns();

            this.gridView.InitNewRow   += gridView_InitNewRow;
            this.addRowButton.Click    += addRowButton_Click;
            this.deleteRowButton.Click += deleteRowButton_Click;
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Private
        //////////////////////////////////////////////////////////////////////////////// Event

        #region 그리드 뷰 신규 행 초기화 하기 - gridView_InitNewRow(sender, e)

        /// <summary>
        /// 그리드 뷰 신규 행 초기화 하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void gridView_InitNewRow(object sender, InitNewRowEventArgs e)
        {
            GridView gridView = sender as GridView;

            gridView.SetRowCellValue(e.RowHandle, gridView.Columns["OrderDate"   ], DateTime.Today            );
            gridView.SetRowCellValue(e.RowHandle, gridView.Columns["RequiredDate"], DateTime.Today.AddDays(10));
            gridView.SetRowCellValue(e.RowHandle, gridView.Columns["ShippedDate" ], DateTime.Today.AddDays(20));
        }

        #endregion
        #region 행 추가 버튼 클릭시 처리하기 - addRowButton_Click(sender, e)

        /// <summary>
        /// 행 추가 버튼 클릭시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void addRowButton_Click(object sender, EventArgs e)
        {
            AddNewRowInGroupMode(this.gridView);

            int rowHandle = this.gridView.GetParentRowHandle(this.gridView.FocusedRowHandle);

            this.gridView.MakeRowVisible(rowHandle);
        }

        #endregion
        #region 행 삭제 버튼 클릭시 처리하기 - deleteRowButton_Click(sender, e)

        /// <summary>
        /// 행 삭제 버튼 클릭시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void deleteRowButton_Click(object sender, EventArgs e)
        {
            this.gridView.DeleteRow(this.gridView.FocusedRowHandle);
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////// Function

        #region 그룹 모드에서 신규 행 추가하기 - AddNewRowInGroupMode(gridView)

        /// <summary>
        /// 그룹 모드에서 신규 행 추가하기
        /// </summary>
        /// <param name="gridView">그리드 뷰</param>
        private void AddNewRowInGroupMode(GridView gridView)
        {
            int rowHandle = gridView.GetDataRowHandleByGroupRowHandle(gridView.FocusedRowHandle);

            object [] groupValueArray = null;

            int groupColumnCount = gridView.GroupedColumns.Count;

            if(groupColumnCount > 0)
            {
                groupValueArray = new object[groupColumnCount];

                for(int i = 0; i < groupColumnCount; i++)
                {
                    groupValueArray[i] = gridView.GetRowCellValue(rowHandle, gridView.GroupedColumns[i]);
                }
            }

            gridView.AddNewRow();

            int newRowHandle = gridView.FocusedRowHandle;

            object newRow = gridView.GetRow(newRowHandle);

            if(groupColumnCount > 0)
            {
                for(int i = 0; i < groupColumnCount; i++)
                {
                    gridView.SetRowCellValue(newRowHandle, gridView.GroupedColumns[i], groupValueArray[i]);
                }
            }

            gridView.UpdateCurrentRow();

            for(int n = 0; n < gridView.DataRowCount; n++)
            {
                if(gridView.GetRow(n).Equals(newRow))
                {
                    gridView.FocusedRowHandle = n;

                   break;
                }
            }
        }

        #endregion
    }
}